# -*- coding: iso-8859-1 -*-

TXEnableDelay = 2e-6
TXEnableValue = 0b0001	# TTL line blanking RF amplifier (bit 0)
TXPulseValue  = 0b0010	# TTL line triggering RF pulses  (bit 1)
ADCSensitivity = 2	# voltage span for ADC

def experiment(): # solid echo (quadrupolar echo) experiment

  # set up acquisition parameters:
    pars = {}
    pars['P90'] = 1.7e-6	# 90-degree pulse length (s)
    pars['SF'] = 338.7e6	# spectrometer frequency (Hz)
    pars['O1'] = -60e3	 	# offset from SF (Hz)
    pars['SW'] = 200e3		# spectral window (Hz)
    pars['SI'] = 1*256		# number of acquisition points
    pars['NS'] = 8		# number of scans
    pars['DS'] = 0		# number of dummy scans
    pars['RD'] = 3		# delay between scans (s)
    pars['TAU'] = 13e-6		# echo delay (s)
    pars['D4'] = 2e-6		# echo pre-acquisition delay (s)
    pars['PHA'] = 0		# receiver phase (degree)
    pars['DATADIR'] = '/home/fprak/Students/'	# data directory
    pars['OUTFILE'] = None			# output file name

  # specify a variable parameter (optional):
    pars['VAR_PAR'] = None	# variable parameter name (a string)   
    start = 20e-6		# starting value
    stop = 100e-6		# end value
    steps = 12			# number of values
    log_scale = False		# log scale flag 
    stag_range = False		# staggered range flag

  # check parameters for safety: 
    if pars['PHA'] < 0:
        pars['PHA'] = 360 + pars['PHA']

    if pars['P90'] > 20e-6:
        raise Exception("Pulse too long!!!")

    # check whether a variable parameter is named: 
    var_key = pars.get('VAR_PAR')
    if var_key == 'P90' and (start > 20e-6 or stop > 20e-6):
        raise Exception("Pulse too long!!!")

    if pars['NS']%8 != 0:
        pars['NS'] = int(round(pars['NS'] / 8) + 1) * 8 
        print 'Number of scans changed to ',pars['NS'],' due to phase cycling' 

  # start the experiment:
    if var_key: 
        # this is an arrayed experiment:
        if log_scale:
            array = log_range(start,stop,steps)
        else:
            array = lin_range(start,stop,steps)

        if stag_range:
            array = staggered_range(array, size = 2)

        # estimate the experiment time:
        if var_key == 'TAU':
            seconds = (sum(array)*2 + pars['RD'] * steps) * (pars['NS'] + pars['DS'])
        elif var_key == 'RD':
            seconds = (sum(array) + pars['TAU']*2 * steps) * (pars['NS'] + pars['DS'])
        else:
            seconds = (pars['TAU']*2 + pars['RD']) * steps * (pars['NS']+ pars['DS'])
        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        print '%s%02d:%02d:%02d' % ('Experiment time estimated: ', h, m, s)

        # loop for a variable parameter:
        for index, pars[var_key] in enumerate(array):
            print 'Arrayed experiment for '+var_key+': run = '+str(index+1)+\
                             ' out of '+str(array.size)+': value = '+str(pars[var_key])
            # loop for accumulation:
            for run in xrange(pars['NS']+pars['DS']):
                yield hahn_experiment(pars, run)
            synchronize()
    else:
       # estimate the experiment time:
        seconds = (pars['TAU']*2 + pars['RD']) * (pars['NS']+ pars['DS'])
        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        print '%s%02d:%02d:%02d' % ('Experiment time estimated: ', h, m, s)

        # loop for accumulation:
        for run in xrange(pars['NS']+pars['DS']):
            yield hahn_experiment(pars, run)       


# the pulse program:

def hahn_experiment(pars, run): 
    e=Experiment()

    dummy_scans = pars.get('DS')
    if dummy_scans:
        run -= dummy_scans

    pars['PROG'] = 'hahn_experiment'

    # phase lists [from Tecmag's pulse sequence]:
    pars['PH1'] = [ 0, 180,   0, 180,  90, 270,  90, 270]	# 90-degree pulse
    pars['PH3'] = [ 0,   0, 180, 180, 270, 270,  90,  90]	# 180-degree pulse
    pars['PH2'] = [ 0, 180,   0, 180,  90, 270,  90, 270]	# receiver

    # read in variables:
    P90 = pars['P90']
    P180 = pars['P90']*2
    SF = pars['SF']
    O1 = pars['O1'] 
    RD =  pars['RD']
    TAU = pars['TAU']
    D4 = pars['D4']
    PH1 = pars['PH1'][run%len(pars['PH1'])]
    PH3 = pars['PH3'][run%len(pars['PH3'])]
    PH2 = pars['PH2'][run%len(pars['PH2'])]
    PHA = pars['PHA']

    # set sampling parameters:
    SI = pars['SI']
    SW = pars['SW']
    while SW <= 10e6 and SI < 256*1024:
        SI *= 2
        SW *= 2

    # run the pulse sequence: 
    e.wait(RD)						# delay between scans
    e.set_frequency(SF+O1, phase=PH1)			# set frequency and phase for 1st RF pulse
    
    e.ttl_pulse(TXEnableDelay, value=TXEnableValue)	# enable RF amplifier
    e.ttl_pulse(P90, value=TXEnableValue|TXPulseValue)	# apply 1st 90-degree pulse
    
    e.wait(TAU-P90/2-P180/2-TXEnableDelay)		# wait for TAU
    e.set_phase(PH3)					# set phase for 2nd 90-degree pulse
    
    e.ttl_pulse(TXEnableDelay, value=TXEnableValue)	# enalble RF amplifier
    e.ttl_pulse(P180, value=TXEnableValue|TXPulseValue)	# apply 2nd 90-degree pulse
    
    e.set_phase(PHA)					# set phase for receiver
    e.wait(TAU-P180/2+D4)				# wait for TAU       
    e.record(SI, SW, sensitivity=ADCSensitivity)	# acquire echo points

    # write the experiment parameters:  
    for key in pars.keys():
        e.set_description(key, pars[key])	# pulse sequence parameters
    e.set_description('run', run)		# current scan
    e.set_description('rec_phase', -PH2)	# current receiver phase

    return e